@@ -178,6 +178,18 @@ void BugzillaManager::attachTextToReport(const QString & text, const QString & f
     attachToReport(request.postData(), request.boundary());
 }
 
+void BugzillaManager::addCommentToReport(const QString & comment, int bugNumber, bool addToCC)
+{
+    KUrl addCommentJobUrl = KUrl(QString(m_bugTrackerUrl) +
+                                            QString(showBugUrl).arg(bugNumber));
+    addCommentJobUrl.addQueryItem("drkonqi_comment", comment);
+    addCommentJobUrl.addQueryItem("drkonqi_addCC", addToCC ? "1" : "0");
+    
+    KIO::Job * addCommentSubJob = KIO::storedGet(addCommentJobUrl, KIO::Reload, KIO::HideProgressInfo);
+
+    connect(addCommentSubJob, SIGNAL(finished(KJob*)) , this, SLOT(addCommentSubJobFinished(KJob*)));
+}
+    
 void BugzillaManager::addMeToCC(int bugNumber)
 {
     KUrl addCCJobUrl = KUrl(QString(m_bugTrackerUrl) +
@@ -266,7 +278,9 @@ void BugzillaManager::fetchBugJobFinished(KJob* job)
         if (parser->isValid()) {
             emit bugReportFetched(report, job->parent());
         } else {
-            emit bugReportError(i18nc("@info","Invalid bug report: corrupted data"), job->parent());
+            emit bugReportError(i18nc("@info","Invalid report information (malformed data). This "
+                                      "could mean that the bug report does not exist, or the "
+                                      "bug tracking site is experiencing a problem."), job->parent());
         }
 
         delete parser;
@@ -302,6 +316,7 @@ void BugzillaManager::sendReportJobFinished(KJob * job)
     if (!job->error()) {
         KIO::StoredTransferJob * sendJob = (KIO::StoredTransferJob *)job;
         QString response = sendJob->data();
+        response.remove('\r'); response.remove('\n');
 
         QRegExp reg("<title>Bug (\\d+) Submitted</title>");
         int pos = reg.indexIn(response);
@@ -311,11 +326,10 @@ void BugzillaManager::sendReportJobFinished(KJob * job)
         } else {
             QString error;
 
-            QRegExp reg("<td id=\"error_msg\" class=\"throw_error\">(.+)</td>");
-            response.remove('\r'); response.remove('\n');
-            pos = reg.indexIn(response);
-            if (pos != -1) {
-                error = reg.cap(1).trimmed();
+            QString errorMessage = getErrorMessage(response, false);
+            if (!errorMessage.isEmpty())
+            {
+                error = errorMessage;
             } else {
                 QString illegalPlatformMessage = "Bugzilla has suffered an internal error. "
                 "Please save this page and send";
@@ -356,19 +370,10 @@ void BugzillaManager::attachToReportJobFinished(KJob * job)
             int bug_id = reg.cap(2).toInt();
             emit attachToReportSent(attach_id, bug_id);
         } else {
-            QString reason;
-
-            QRegExp reg("<td id=\"error_msg\" class=\"throw_error\">(.+)</td>");
-            response.remove('\r'); response.remove('\n');
-            pos = reg.indexIn(response);
-            if (pos != -1) {
-                reason = reg.cap(1).trimmed();
-            } else {
-                reason = i18nc("@info","Unknown error");
-            }
+            QString errorMessage = getErrorMessage(response);
 
             QString error = i18nc("@info", "Error while attaching the data to the bug report: %1", 
-                                  reason);
+                                  errorMessage);
             emit attachToReportError(error);
         }
     } else {
@@ -376,6 +381,69 @@ void BugzillaManager::attachToReportJobFinished(KJob * job)
     }
 }
 
+void BugzillaManager::addCommentSubJobFinished(KJob * job)
+{
+    if (!job->error()) {
+        KIO::StoredTransferJob * checkJob = (KIO::StoredTransferJob *)job;
+        QString response = checkJob->data();
+        response.remove('\r'); response.remove('\n');
+
+        //Parse query values
+        QString token = getToken(response);
+        const QString bug_id = checkJob->url().queryItem("id");
+        const QString comment = checkJob->url().queryItem("drkonqi_comment");
+        const QString addCCString = checkJob->url().queryItem("drkonqi_addCC");
+        bool addToCC = (addCCString == QLatin1String("1"));
+
+        if (!bug_id.isEmpty() && !comment.isEmpty()) {
+            QByteArray postData;
+            postData += 
+                QByteArray("id=") +  QUrl::toPercentEncoding(bug_id) +
+                QByteArray("&token=") +  QUrl::toPercentEncoding(token) +
+                QByteArray("&comment=") +  QUrl::toPercentEncoding(comment);
+            if (addToCC) {
+                postData += QByteArray("&addselfcc=on");
+            }        
+                
+            KIO::Job * addCommentJob =
+                KIO::storedHttpPost(postData, KUrl(QString(m_bugTrackerUrl) +
+                                                                QString(addInformationUrl)),
+                                KIO::HideProgressInfo);
+            connect(addCommentJob, SIGNAL(finished(KJob*)) , this, SLOT(addCommentJobFinished(KJob*)));
+            addCommentJob->addMetaData("content-type", "Content-Type: application/x-www-form-urlencoded");
+        } else {
+            emit addCommentError(i18nc("@info","Missing bug ID or comment in the query. Unknown error"));
+        }
+
+    } else {
+        emit addCommentError(job->errorString());
+    }
+}
+
+void BugzillaManager::addCommentJobFinished(KJob * job)
+{
+    if (!job->error()) {
+        KIO::StoredTransferJob * addCommentJob = (KIO::StoredTransferJob *)job;
+        QString response = addCommentJob->data();
+        response.remove('\r'); response.remove('\n');
+
+        QRegExp reg("<title>Bug (.+) processed</title>");
+        int pos = reg.indexIn(response);
+        if (pos != -1) {
+            int bug_id = reg.cap(1).toInt();
+            emit addCommentFinished(bug_id);
+        } else {
+            QString errorMessage = getErrorMessage(response);
+
+            QString error = i18nc("@info", "Error while adding a new comment into the bug report: %1", 
+                                  errorMessage);
+            emit addCommentError(error);
+        }
+    } else {
+        emit addCommentError(job->errorString());
+    }  
+}
+
 void BugzillaManager::addMeToCCSubJobFinished(KJob * job)
 {
     if (!job->error()) {
@@ -384,35 +452,25 @@ void BugzillaManager::addMeToCCSubJobFinished(KJob * job)
         response.remove('\r'); response.remove('\n');
 
         //Parse bugzilla token
-        QString token;
-
-        QString searchToken = QLatin1String("<input type=\"hidden\" name=\"token\" value=\"");
-        int index1 = response.indexOf(searchToken);
-        int index2 = response.indexOf(">", index1);
-
-        if (index1 != -1 && index2 != -1) {
-            index1 += searchToken.length();
-            index2 -= 1;
-
-            token = response.mid(index1, index2-index1);
-        }
+        QString token = getToken(response);
 
         const QString bug_id = checkJob->url().queryItem("id");
 
         if (!bug_id.isEmpty()) {
             //Send add To CC job confirmation
             QByteArray postData;
-            postData += QByteArray("id=") + bug_id.toUtf8() + QByteArray("&addselfcc=on") +
-            QByteArray("&token=") + token.toUtf8();
+            postData += QByteArray("id=") + QUrl::toPercentEncoding(bug_id) +
+            QByteArray("&addselfcc=on") +
+            QByteArray("&token=") + QUrl::toPercentEncoding(token);
 
             KIO::Job * addCCJob =
                 KIO::storedHttpPost(postData, KUrl(QString(m_bugTrackerUrl) +
                                                                 QString(addInformationUrl)),
                                 KIO::HideProgressInfo);
-            connect(addCCJob, SIGNAL(finished(KJob*)) , this, SLOT(addMeToCCJobFinished(KJob*)));
             addCCJob->addMetaData("content-type", "Content-Type: application/x-www-form-urlencoded");
+            connect(addCCJob, SIGNAL(finished(KJob*)) , this, SLOT(addMeToCCJobFinished(KJob*)));
         } else {
-            emit addMeToCCError(i18nc("@info","Unknown error"));
+            emit addMeToCCError(i18nc("@info","Missing bug ID in the query. Unknown error"));
         }
 
     } else {
@@ -433,19 +491,10 @@ void BugzillaManager::addMeToCCJobFinished(KJob * job)
             int bug_id = reg.cap(1).toInt();
             emit addMeToCCFinished(bug_id);
         } else {
-            QString reason;
-
-            QRegExp reg("<td id=\"error_msg\" class=\"throw_error\">(.+)</td>");
-            response.remove('\r'); response.remove('\n');
-            pos = reg.indexIn(response);
-            if (pos != -1) {
-                reason = reg.cap(1).trimmed();
-            } else {
-                reason = i18nc("@info","Unknown error");
-            }
+            QString errorMessage = getErrorMessage(response);
 
             QString error = i18nc("@info", "Error while adding yourself to the CC list: %1", 
-                                  reason);
+                                  errorMessage);
 
             emit addMeToCCError(error);
         }
@@ -467,7 +516,7 @@ void BugzillaManager::checkVersionJobFinished(KJob * job)
             return;
         }
 
-        //Be ware: Quick'n'dirty HTML parsing ....
+        //Warning!: Quick'n'dirty HTML parsing ....
 
         const QString firstSelect = "<select name=\"product\"";
         const QString searchString = "<option value=\"" + product;
@@ -575,6 +624,40 @@ void BugzillaManager::attachToReport(const QByteArray & data, const QByteArray &
                            "Content-Type: multipart/form-data; boundary=" + boundary);   
 }
 
+QString BugzillaManager::getToken(const QString & response)
+{
+    QString token;
+
+    QString searchToken = QLatin1String("<input type=\"hidden\" name=\"token\" value=\"");
+    int index1 = response.indexOf(searchToken);
+    int index2 = response.indexOf(">", index1);
+
+    if (index1 != -1 && index2 != -1) {
+        index1 += searchToken.length();
+        index2 -= 1;
+        token = response.mid(index1, index2-index1);
+    }
+
+    return token;
+}
+
+QString BugzillaManager::getErrorMessage(const QString & response, bool fallbackMessage)
+{
+    QString errorMessage;
+
+    QRegExp reg("<td id=\"error_msg\" class=\"throw_error\">(.+)</td>");
+    int pos = reg.indexIn(response);
+    if (pos != -1) {
+        errorMessage = reg.cap(1).trimmed();
+    } else {
+        if (fallbackMessage) {
+            errorMessage = i18nc("@info","Unknown error");
+        }
+    }
+
+    return errorMessage;
+}
+
 QByteArray BugzillaManager::generatePostDataForReport(BugReport report) const
 {
     QByteArray postData =
@@ -694,6 +777,7 @@ BugReport BugReportXMLParser::parse()
             report.setPriority(getSimpleValue("priority"));
             report.setBugSeverity(getSimpleValue("bug_severity"));
             report.setMarkedAsDuplicateOf(getSimpleValue("dup_id"));
+            report.setVersionFixedIn(getSimpleValue("cf_versionfixedin"));
 
             //Parse full content + comments
             QStringList m_commentList;
