@@ -58,13 +58,21 @@ BugzillaDuplicatesPage::BugzillaDuplicatesPage(ReportAssistantDialog * parent):
     header->setResizeMode(0, QHeaderView::ResizeToContents);
     header->setResizeMode(1, QHeaderView::Interactive);
 
+    //Create manual bug report entry (first one)
     QTreeWidgetItem * customBugItem = new QTreeWidgetItem(
-        QStringList() << i18nc("@item:intable custom bug report number", "Custom")
+        QStringList() << i18nc("@item:intable custom/manaul bug report number", "Manual")
                       << i18nc("@item:intable custom bug report number description", 
                                                             "Manually enter a bug report ID"));
     customBugItem->setData(0, Qt::UserRole, QLatin1String("custom"));
     customBugItem->setIcon(1, KIcon("edit-rename"));
 
+    QString helpMessage = i18nc("@info:tooltip / whatsthis",
+                                "Select this option to manually load a specific bug report");
+    customBugItem->setToolTip(0, helpMessage);
+    customBugItem->setToolTip(1, helpMessage);
+    customBugItem->setWhatsThis(0, helpMessage);
+    customBugItem->setWhatsThis(1, helpMessage);
+
     ui.m_bugListWidget->addTopLevelItem(customBugItem);
 
     m_searchMoreGuiItem = KGuiItem2(i18nc("@action:button", "Search for more reports"),
@@ -163,11 +171,11 @@ bool BugzillaDuplicatesPage::showNextPage()
         //The user didn't selected any possible duplicate nor a report to attach the new info.
         //Double check this, we need to reduce the duplicate count.
         KGuiItem noDuplicatesButton;
-        noDuplicatesButton.setText("There are no real duplicates");
+        noDuplicatesButton.setText(i18n("There are no real duplicates"));
         noDuplicatesButton.setIcon(KIcon("dialog-cancel"));
 
         KGuiItem letMeCheckMoreReportsButton;
-        letMeCheckMoreReportsButton.setText("Let me check more reports");
+        letMeCheckMoreReportsButton.setText(i18n("Let me check more reports"));
         letMeCheckMoreReportsButton.setIcon(KIcon("document-preview"));
 
         if (KMessageBox::questionYesNo(this,
@@ -209,7 +217,7 @@ void BugzillaDuplicatesPage::performSearch()
         endDateStr = QLatin1String("Now");
     }
 
-#if true
+#if 1
     BugReport report = reportInterface()->newBugReportTemplate();
     bugzillaManager()->searchBugs(reportInterface()->relatedBugzillaProducts(),
                                   report.bugSeverity(), startDateStr, endDateStr, 
@@ -300,7 +308,7 @@ void BugzillaDuplicatesPage::searchFinished(const BugMapList & list)
                 } else if (bug.value("resolution") == QLatin1String("WORKSFORME")) {
                     customStatusString = i18nc("@info/plain bug resolution", "[Non-reproducible]");
                 } else if (bug.value("resolution") == QLatin1String("DUPLICATE")) {
-                    customStatusString = i18nc("@info/plain bug resolution", "[Already reported]");
+                    customStatusString = i18nc("@info/plain bug resolution", "[Duplicate report]");
                 } else if (bug.value("resolution") == QLatin1String("INVALID")) {
                     customStatusString = i18nc("@info/plain bug resolution", "[Invalid]");
                 } else if (bug.value("resolution") == QLatin1String("DOWNSTREAM")
@@ -314,7 +322,12 @@ void BugzillaDuplicatesPage::searchFinished(const BugMapList & list)
             title = customStatusString + ' ' + bug["short_desc"];
 
             QStringList fields = QStringList() << bug["bug_id"] << title;
-            ui.m_bugListWidget->addTopLevelItem(new QTreeWidgetItem(fields));
+
+            QTreeWidgetItem * item = new QTreeWidgetItem(fields);
+            item->setToolTip(0, bug["short_desc"]);
+            item->setToolTip(1, bug["short_desc"]);
+
+            ui.m_bugListWidget->addTopLevelItem(item);
         }
 
         ui.m_bugListWidget->sortItems(0 , Qt::DescendingOrder);
@@ -477,10 +490,11 @@ void BugzillaDuplicatesPage::cancelAttachToBugReport()
 BugzillaReportInformationDialog::BugzillaReportInformationDialog(BugzillaDuplicatesPage * parent) :
         KDialog(parent),
         m_parent(parent),
-        m_bugNumber(0)
+        m_bugNumber(0),
+        m_duplicatesCount(0)
 {
     //Create the GUI
-    setButtons(KDialog::Close | KDialog::User1 | KDialog::User2);
+    setButtons(KDialog::Close | KDialog::User1);
     setDefaultButton(KDialog::Close);
     setCaption(i18nc("@title:window","Bug Description"));
     
@@ -493,20 +507,15 @@ BugzillaReportInformationDialog::BugzillaReportInformationDialog(BugzillaDuplica
                                   i18nc("@info:tooltip", "Use this button to retry "
                                                   "loading the bug report.")));
     connect(ui.m_retryButton, SIGNAL(clicked()), this, SLOT(reloadReport()));
-    
-    setButtonGuiItem(KDialog::User2, 
-                KGuiItem2(i18nc("@action:button", "Add as a possible duplicate"),
-                    KIcon("list-add"), i18nc("@info:tooltip", "Use this button to mark your "
-                    "crash as related to the currently shown bug report. This will help "
-                    "the KDE developers to determine whether they are duplicates or not.")));
-    connect(this, SIGNAL(user2Clicked()) , this, SLOT(mayBeDuplicateClicked()));
 
-    setButtonGuiItem(KDialog::User1, 
-                KGuiItem2(i18nc("@action:button", "Attach to this report (Advanced)"),
-                    KIcon("mail-attachment"), i18nc("@info:tooltip", "Use this button to attach "
-                    "your crash information to this report; only if you are really sure this is "
-                    "the same crash.")));
-    connect(this, SIGNAL(user1Clicked()) , this, SLOT(attachToBugReportClicked()));
+    setButtonGuiItem(KDialog::User1,
+                KGuiItem2(i18nc("@action:button", "Suggest this crash is related"),
+                    KIcon("list-add"), i18nc("@info:tooltip", "Use this button to suggest that "
+                                             "the crash you experienced is related to this bug "
+                                             "report")));
+    connect(this, SIGNAL(user1Clicked()) , this, SLOT(relatedReportClicked()));
+    
+    connect(ui.m_showOwnBacktraceCheckBox, SIGNAL(toggled(bool)), this, SLOT(toggleShowOwnBacktrace(bool)));
 
     //Connect bugzillalib signals
     connect(m_parent->bugzillaManager(), SIGNAL(bugReportFetched(BugReport, QObject *)),
@@ -544,13 +553,11 @@ void BugzillaReportInformationDialog::showBugReport(int bugNumber)
     m_parent->bugzillaManager()->fetchBugReport(m_bugNumber, this);
 
     button(KDialog::User1)->setEnabled(false);
-    button(KDialog::User2)->setEnabled(false);
 
     ui.m_infoBrowser->setText(i18nc("@info:status","Loading..."));
     ui.m_infoBrowser->setEnabled(false);
     
-    ui.m_linkLabel->setText(i18nc("@info","<link url='%1'>Bug report page at the KDE bug "
-                                              "tracking system</link>",
+    ui.m_linkLabel->setText(i18nc("@info","<link url='%1'>Report's webpage</link>",
                                     m_parent->bugzillaManager()->urlForBug(m_bugNumber)));
 
     ui.m_statusWidget->setBusy(i18nc("@info:status","Loading information about bug "
@@ -558,7 +565,17 @@ void BugzillaReportInformationDialog::showBugReport(int bugNumber)
                                             m_bugNumber,
                                             QLatin1String(KDE_BUGZILLA_SHORT_URL)));
                                             
-    ui.m_backtraceBrowser->setPlainText(m_parent->reportInterface()->backtrace());
+    ui.m_backtraceBrowser->setPlainText(
+                    i18nc("@info/plain","Backtrace of the crash I experienced:\n\n") +
+                    m_parent->reportInterface()->backtrace());
+
+    KConfigGroup config(KGlobal::config(), "BugzillaReportInformationDialog");
+    bool showOwnBacktrace = config.readEntry("ShowOwnBacktrace", false);
+    ui.m_showOwnBacktraceCheckBox->setChecked(showOwnBacktrace);
+    if (!showOwnBacktrace) { //setChecked(false) will not emit toggled(false)
+        toggleShowOwnBacktrace(false);
+    }
+
     show();
 }
 
@@ -567,7 +584,7 @@ void BugzillaReportInformationDialog::bugFetchFinished(BugReport report, QObject
     if (jobOwner == this && isVisible()) {
         if (report.isValid()) {
 
-            //Resolve duplicates
+            //Handle duplicate state
             QString duplicate = report.markedAsDuplicateOf();
             if (!duplicate.isEmpty()) {
                 bool ok = false;
@@ -596,22 +613,27 @@ void BugzillaReportInformationDialog::bugFetchFinished(BugReport report, QObject
                 }
             }
 
-            //Generate html for comments
+            //Generate html for comments (with proper numbering)
             QString comments;
             QStringList commentList = report.comments();
             for (int i = 0; i < commentList.count(); i++) {
                 QString comment = commentList.at(i);
                 comment.replace('\n', "<br />");
-                comments += "<p><hr />" + comment + "</p>";
+                comments += i18nc("comment $number to use as subtitle", "<h4>Comment %1:</h4>", (i+1))
+                                  + "<p>" + comment + "</p><hr />";
             }
 
+            //Count duplicates (simple check)
+            m_duplicatesCount = comments.count("has been marked as a duplicate of this bug.");
+
             //Generate a non-geek readable status
             QString customStatusString;
-            if (report.bugStatus() == QLatin1String("UNCONFIRMED")
-                || report.bugStatus() == QLatin1String("NEW")
+            if (report.bugStatus() == QLatin1String("UNCONFIRMED")) {
+                customStatusString = i18nc("@info bug status", "Opened (Unconfirmed)");
+            } else if (report.bugStatus() == QLatin1String("NEW")
                 || report.bugStatus() == QLatin1String("REOPENED")
                 || report.bugStatus() == QLatin1String("ASSIGNED")) {
-                customStatusString = i18nc("@info bug status", "Opened");
+                customStatusString = i18nc("@info bug status", "Opened (Unfixed)");
             } else if (report.bugStatus() == QLatin1String("RESOLVED")
                 || report.bugStatus() == QLatin1String("VERIFIED")
                 || report.bugStatus() == QLatin1String("CLOSED")) {
@@ -622,7 +644,8 @@ void BugzillaReportInformationDialog::bugFetchFinished(BugReport report, QObject
                 } else if (report.resolution() == QLatin1String("WORKSFORME")) {
                     customResolutionString = i18nc("@info bug resolution", "Non-reproducible");
                 } else if (report.resolution() == QLatin1String("DUPLICATE")) {
-                    customResolutionString = i18nc("@info bug resolution", "Already reported");
+                    customResolutionString = i18nc("@info bug resolution", "Duplicate report "
+                                                   "(Already reported before)");
                 } else if (report.resolution() == QLatin1String("INVALID")) {
                     customResolutionString = i18nc("@info bug resolution", "Not a valid report/crash");
                 } else if (report.resolution() == QLatin1String("DOWNSTREAM") 
@@ -631,7 +654,7 @@ void BugzillaReportInformationDialog::bugFetchFinished(BugReport report, QObject
                                                 "in the KDE's Applications or libraries");
                     m_closedStateString = i18nc("@info bug resolution", "the bug is caused by a "
                                                 "problem in an external application or library, or "
-                                                "by a distribution or packaging issue.");
+                                                "by a distribution or packaging issue");
                 } else {
                     customResolutionString = report.resolution();
                 }
@@ -641,24 +664,54 @@ void BugzillaReportInformationDialog::bugFetchFinished(BugReport report, QObject
             } else if (report.bugStatus() == QLatin1String("NEEDSINFO")) {
                 customStatusString = i18nc("@info bug status", "Temporarily closed, because of a lack "
                                             "of information");
-            } else {
+            } else { //Fallback to other raw values
                 customStatusString = QString("%1 (%2)").arg(report.bugStatus(), report.resolution());
             }
-            
+
+            //Generate notes
+            QString notes = i18n("<p><note>The bug report's title is often written by its reporter "
+                                 "and may not reflect the bug's nature, root cause or other visible "
+                                 "symptoms you could use to compare to your crash. Please read the "
+                                 "complete report and all the comments below.</note></p>");
+
+            if (m_duplicatesCount >= 10) { //Consider a possible mass duplicate crash
+                notes += i18np("<p><note>This bug report has %1 duplicate report. That means this "
+                               "is probably a <strong>common crash</strong>. <i>Please consider only "
+                               "adding a comment or a note if you can provide new valuable "
+                               "information which was not already mentioned.</i></note></p>",
+                               "<p><note>This bug report has %1 duplicate reports. That means this "
+                               "is probably a <strong>common crash</strong>. <i>Please consider only "
+                               "adding a comment or a note if you can provide new valuable "
+                               "information which was not already mentioned.</i></note></p>",
+                              m_duplicatesCount);
+            }
+
+            //A manually entered bug ID could represent a normal bug
+            if (report.bugSeverity() != QLatin1String("crash")
+                && report.bugSeverity() != QLatin1String("major")
+                && report.bugSeverity() != QLatin1String("grave")
+                && report.bugSeverity() != QLatin1String("critical"))
+            {
+                notes += i18n("<p><note>This bug report is not about a crash or about any other "
+                              "critical bug.</note></p>");
+            }
+
+            //Generate HTML text
             QString text =
-                i18nc("@info bug report label and value",
-                                "<h2>%1</h2>", report.shortDescription()) +
-                i18nc("@info bug report label and value",
-                                "<h3>Bug Report Status: %1</h3>", customStatusString) +
-                i18nc("@info bug report label and value",
-                                "<p><strong>Product:</strong> %1 (%2)</p>",
+                i18nc("@info bug report title (quoted)",
+                                "<h3>\"%1\"</h3>", report.shortDescription()) +
+                notes +
+                i18nc("@info bug report status",
+                                "<h4>Bug Report Status: %1</h4>", customStatusString) +
+                i18nc("@info bug report product and component",
+                                "<h4>Affected Component: %1 (%2)</h4>",
                                 report.product(), report.component()) +
-                i18nc("@info bug report label and value",
-                                "<h2>Description of the bug</h2><p>%1</p>",
+                i18nc("@info bug report description",
+                                "<h3>Description of the bug</h3><p>%1</p>",
                                 report.description().replace('\n', "<br />"));
 
             if (!comments.isEmpty()) {
-                text += i18nc("@label:textbox bug report label and value", 
+                text += i18nc("@label:textbox bug report comments (already formatted)", 
                                 "<h2>Additional Comments</h2>%1", comments);
             }
 
@@ -666,80 +719,41 @@ void BugzillaReportInformationDialog::bugFetchFinished(BugReport report, QObject
             ui.m_infoBrowser->setEnabled(true);
             
             button(KDialog::User1)->setEnabled(true);
-            button(KDialog::User2)->setEnabled(true);
 
-            ui.m_statusWidget->setIdle(i18nc("@info:status", "Showing report <numid>%1</numid>",
+            ui.m_statusWidget->setIdle(i18nc("@info:status", "Showing bug <numid>%1</numid>",
                                                             report.bugNumberAsInt()));
         } else {
-            bugFetchError(i18nc("@info","Invalid report data"), this);
+            bugFetchError(i18nc("@info", "Invalid report information (malformed data). This could "
+                                "mean that the bug report does not exist, or the bug tracking site "
+                                "is experiencing a problem."), this);
         }
     }
 }
 
-int BugzillaReportInformationDialog::promptAboutAlreadyClosedReport()
+void BugzillaReportInformationDialog::markAsDuplicate()
 {
-    KGuiItem sameBugDoNotReportButton;
-    sameBugDoNotReportButton.setText("It is the same bug. Do not file my report (Close).");
-    sameBugDoNotReportButton.setIcon(KIcon("edit-copy"));
-
-    KGuiItem userIsNotSureProceedButton;
-    userIsNotSureProceedButton.setText("I cannot be sure. Proceed with reporting the bug.");
-    userIsNotSureProceedButton.setIcon(KIcon("go-next"));
-
-    int ret = KMessageBox::questionYesNoCancel(this,
-        i18nc("@info messagebox question. %1 is the close state explanation","This report is marked as "
-        "\"closed\" because %1. If you are sure your crash is the same, then adding further details "
-        "or creating a new report will be useless and will waste developers' time. "
-        "Can you be sure this is the same as your crash?", m_closedStateString),
-        i18nc("@title:window", "This report is already closed"), sameBugDoNotReportButton,
-                   userIsNotSureProceedButton, KStandardGuiItem::cancel(), QString(),
-                                               KMessageBox::Dangerous | KMessageBox::Notify);
-
-    return ret;
-}
-
-void BugzillaReportInformationDialog::mayBeDuplicateClicked()
-{
-    if (!m_closedStateString.isEmpty()) {
-        int ret = promptAboutAlreadyClosedReport();
-        if (ret == KMessageBox::Yes) { // "Same bug report, close"
-            //Ask to close the report assistant dialog
-            hide();
-            m_parent->assistant()->close();
-            return;
-        } else if (ret == KMessageBox::Cancel) {
-            return;
-        }
-    }
     emit possibleDuplicateSelected(m_bugNumber);
     hide();
 }
 
-void BugzillaReportInformationDialog::attachToBugReportClicked()
+void BugzillaReportInformationDialog::attachToBugReport()
 {
-    if (!m_closedStateString.isEmpty()) {
-        int ret = promptAboutAlreadyClosedReport();
-        if (ret == KMessageBox::Yes) { // "Same bug report, close"
-            //Ask to close the report assistant dialog
-            hide();
-            m_parent->assistant()->close();
-            return;
-        } else if (ret == KMessageBox::Cancel) {
-            return;
-        }
-        emit attachToBugReportSelected(m_bugNumber);
-        hide();
-    } else {
-        if (KMessageBox::questionYesNo(this,
-               i18nc("@info","If you want to attach new information to an existing bug report you need "
-               "to be sure that they refer to the same crash.<nl />Are you sure you want to attach "
-               "your report to bug <numid>%1</numid> ?", m_bugNumber),
-               i18nc("@title:window","Attach the information to bug <numid>%1</numid>", m_bugNumber))
-                                            == KMessageBox::Yes) {
-            emit attachToBugReportSelected(m_bugNumber);
-            hide();
-        }
-    }
+    emit attachToBugReportSelected(m_bugNumber);
+    hide();
+}
+
+void BugzillaReportInformationDialog::cancelAssistant()
+{
+    m_parent->assistant()->close();
+    hide();
+}
+
+void BugzillaReportInformationDialog::relatedReportClicked()
+{
+    BugzillaReportConfirmationDialog * confirmation =
+        new BugzillaReportConfirmationDialog(m_bugNumber, (m_duplicatesCount >= 10),
+                                             m_closedStateString, this);
+    confirmation->show();
 }
 
 void BugzillaReportInformationDialog::bugFetchError(QString err, QObject * jobOwner)
@@ -749,11 +763,149 @@ void BugzillaReportInformationDialog::bugFetchError(QString err, QObject * jobOw
                                         "<message>%1.</message><nl/>"
                                         "Please wait some time and try again.", err));
         button(KDialog::User1)->setEnabled(false);
-        button(KDialog::User2)->setEnabled(false);
         ui.m_infoBrowser->setText(i18nc("@info","Error fetching the bug report"));
         ui.m_statusWidget->setIdle(i18nc("@info:status","Error fetching the bug report"));
         ui.m_retryButton->setVisible(true);
     }
 }
 
+void BugzillaReportInformationDialog::toggleShowOwnBacktrace(bool show)
+{
+    QList<int> sizes;
+    if (show) {
+        int size = (ui.m_reportSplitter->sizeHint().width()-ui.m_reportSplitter->handleWidth())/2;
+        sizes << size << size;
+    } else {
+        sizes << ui.m_reportSplitter->sizeHint().width() << 0; //Hide backtrace
+    }
+    ui.m_reportSplitter->setSizes(sizes);
+
+    //Save the current show value
+    KConfigGroup config(KGlobal::config(), "BugzillaReportInformationDialog");
+    config.writeEntry("ShowOwnBacktrace", show);
+}
+
 //END BugzillaReportInformationDialog
+
+//BEGIN BugzillaReportConfirmationDialog
+
+BugzillaReportConfirmationDialog::BugzillaReportConfirmationDialog(int bugNumber, bool commonCrash,
+    QString closedState, BugzillaReportInformationDialog * parent)
+    : KDialog(parent),
+    m_parent(parent),
+    m_showProceedQuestion(false),
+    m_bugNumber(bugNumber)
+{
+    KGlobal::ref();
+    setAttribute(Qt::WA_DeleteOnClose, true);
+    setModal(true);
+
+    QWidget * widget = new QWidget(this);
+    ui.setupUi(widget);
+    setMainWidget(widget);
+
+    //Setup dialog
+    setButtons(KDialog::Ok | KDialog::Cancel);
+    setDefaultButton(KDialog::Cancel);
+    setCaption(i18nc("@title:window", "Related Bug Report"));
+
+    //Setup buttons
+    button(KDialog::Cancel)->setText(i18nc("@action:button", "Cancel (Go back to the report)"));
+    button(KDialog::Ok)->setText(i18nc("@action:button continue with the selected option "
+                                       "and close the dialog", "Continue"));
+    button(KDialog::Ok)->setEnabled(false);
+
+    connect(this, SIGNAL(okClicked()) , this, SLOT(proceedClicked()));
+    connect(this, SIGNAL(cancelClicked()) , this, SLOT(hide()));
+
+    //Set introduction text
+    ui.introLabel->setText(i18n("You are going to mark your crash as related to bug <numid>%1</numid>",
+                             m_bugNumber));
+
+    if (commonCrash) { //Common ("massive") crash
+        m_showProceedQuestion = true;
+        ui.commonCrashIcon->setPixmap(KIcon("edit-bomb").pixmap(22,22));
+    } else {
+        ui.commonCrashLabel->setVisible(false);
+        ui.commonCrashIcon->setVisible(false);
+    }
+
+    if (!closedState.isEmpty()) { //Bug report closed
+        ui.closedReportLabel->setText(
+                     i18nc("@info", "The report is closed because %1. "
+                           "<i>If the crash is the same, adding further information will be useless "
+                           "and will consume developers' time.</i>",
+                           closedState));
+        ui.closedReportIcon->setPixmap(KIcon("document-close").pixmap(22,22));
+        m_showProceedQuestion = true;
+    } else {
+        ui.closedReportLabel->setVisible(false);
+        ui.closedReportIcon->setVisible(false);
+    }
+
+    //Disable all the radio buttons
+    ui.proceedRadioYes->setChecked(false);
+    ui.proceedRadioNo->setChecked(false);
+    ui.markAsDuplicateCheck->setChecked(false);
+    ui.attachToBugReportCheck->setChecked(false);
+
+    connect(ui.buttonGroupProceed, SIGNAL(buttonClicked(int)), this, SLOT(checkProceed()));
+    connect(ui.buttonGroupProceedQuestion, SIGNAL(buttonClicked(int)), this, SLOT(checkProceed()));
+
+    if (!m_showProceedQuestion) {
+        ui.proceedLabel->setEnabled(false);
+        ui.proceedRadioYes->setEnabled(false);
+        ui.proceedRadioNo->setEnabled(false);
+
+        ui.proceedLabel->setVisible(false);
+        ui.proceedRadioYes->setVisible(false);
+        ui.proceedRadioNo->setVisible(false);
+
+        ui.proceedRadioYes->setChecked(true);
+    }
+
+    checkProceed();
+
+    setMinimumSize(QSize(600, 350));
+    setInitialSize(QSize(600, 350));
+}
+
+BugzillaReportConfirmationDialog::~BugzillaReportConfirmationDialog()
+{
+    KGlobal::deref();
+}
+
+void BugzillaReportConfirmationDialog::checkProceed()
+{
+    bool yes = ui.proceedRadioYes->isChecked();
+    bool no = ui.proceedRadioNo->isChecked();
+    
+    //Enable/disable labels and controls
+    ui.areYouSureLabel->setEnabled(yes);
+    ui.markAsDuplicateCheck->setEnabled(yes);
+    ui.attachToBugReportCheck->setEnabled(yes);
+
+    //Enable Continue button if valid options are selected
+    bool possibleDupe = ui.markAsDuplicateCheck->isChecked();
+    bool attach = ui.attachToBugReportCheck->isChecked();
+    bool enableContinueButton = yes ? (possibleDupe || attach) : no;
+    button(KDialog::Ok)->setEnabled(enableContinueButton);
+}
+
+void BugzillaReportConfirmationDialog::proceedClicked()
+{
+    if (ui.proceedRadioYes->isChecked()) {
+        if (ui.markAsDuplicateCheck->isChecked()) {
+            m_parent->markAsDuplicate();
+            hide();
+        } else {
+            m_parent->attachToBugReport();
+            hide();
+        }
+    } else {
+        hide();
+        m_parent->cancelAssistant();
+    }
+}
+
+//END BugzillaReportConfirmationDialog
