@@ -28,7 +28,6 @@
 #include <QtGui/QLayout>
 #include <QtCore/QTimer>
 #include <QtGui/QBoxLayout>
-#include <QFile>
 #include <qdbusconnection.h>
 #include <qdbusmessage.h>
 
@@ -83,7 +82,6 @@ FileTypesView::FileTypesView(QWidget *parent, const QVariantList &)
 
   QHBoxLayout* l = new QHBoxLayout(this);
   QVBoxLayout* leftLayout = new QVBoxLayout();
-  leftLayout->setSpacing(KDialog::spacingHint());
   l->addLayout( leftLayout );
 
   patternFilterLE = new KLineEdit(this);
