@@ -37,6 +37,8 @@
 #include <QDesktopWidget>
 #include <QDBusConnection>
 #include <QDBusConnectionInterface>
+#include <QDBusServiceWatcher>
+#include <QXmlStreamReader>
 #include <kconfiggroup.h>
 
 static const char dbusServiceName[] = "org.freedesktop.Notifications";
@@ -44,7 +46,8 @@ static const char dbusInterfaceName[] = "org.freedesktop.Notifications";
 static const char dbusPath[] = "/org/freedesktop/Notifications";
 
 NotifyByPopup::NotifyByPopup(QObject *parent) 
-  : KNotifyPlugin(parent) , m_animationTimer(0), m_dbusServiceExists(false)
+  : KNotifyPlugin(parent) , m_animationTimer(0), m_dbusServiceExists(false),
+		m_dbusServiceCapCacheDirty(true)
 {
 	QRect screen = QApplication::desktop()->availableGeometry();
 	m_nextPosition = screen.top();
@@ -57,7 +60,11 @@ NotifyByPopup::NotifyByPopup(QObject *parent)
 		slotServiceOwnerChanged(dbusServiceName, QString(), "_"); //connect signals
 
 	// to catch register/unregister events from service in runtime
-	connect(interface, SIGNAL(serviceOwnerChanged(const QString&, const QString&, const QString&)),
+	QDBusServiceWatcher *watcher = new QDBusServiceWatcher(this);
+	watcher->setConnection(QDBusConnection::sessionBus());
+	watcher->setWatchMode(QDBusServiceWatcher::WatchForOwnerChange);
+	watcher->addWatchedService(dbusServiceName);
+	connect(watcher, SIGNAL(serviceOwnerChanged(const QString&, const QString&, const QString&)),
 			SLOT(slotServiceOwnerChanged(const QString&, const QString&, const QString&)));
 }
 
@@ -70,11 +77,11 @@ NotifyByPopup::~NotifyByPopup()
 
 void NotifyByPopup::notify( int id, KNotifyConfig * config )
 {
-	kDebug(300) << id << "  active notifications:" << m_popups.keys() << m_idMap.keys();
+	kDebug() << id << "  active notifications:" << m_popups.keys() << m_idMap.keys();
 
 	if(m_popups.contains(id) || m_idMap.contains(id))
 	{
-		kDebug(300) << "the popup is already shown";
+		kDebug() << "the popup is already shown";
 		finish(id);
 		return;
 	}
@@ -159,7 +166,7 @@ void NotifyByPopup::slotLinkClicked( const QString &adr )
 	unsigned int id=adr.section('/' , 0 , 0).toUInt();
 	unsigned int action=adr.section('/' , 1 , 1).toUInt();
 
-//	kDebug(300) << id << " " << action;
+//	kDebug() << id << " " << action;
         
 	if(id==0 || action==0)
 		return;
@@ -254,56 +261,57 @@ void NotifyByPopup::fillPopup(KPassivePopup *pop,int id,KNotifyConfig * config)
 void NotifyByPopup::slotServiceOwnerChanged( const QString & serviceName,
 		const QString & oldOwner, const QString & newOwner )
 {
-	if(serviceName == dbusServiceName)
+	kDebug() << serviceName << oldOwner << newOwner;
+	// tell KNotify that all existing notifications which it sent
+	// to DBus had been closed
+	foreach (int id, m_idMap.keys())
+		finished(id);
+	m_idMap.clear();
+
+	m_dbusServiceCapCacheDirty = true;
+	m_dbusServiceCapabilities.clear();
+
+	if(newOwner.isEmpty())
 	{
-		kDebug(300) << serviceName << oldOwner << newOwner;
-		// tell KNotify that all existing notifications which it sent
-		// to DBus had been closed
-		foreach (int id, m_idMap.keys())
-			finished(id);
-		m_idMap.clear();
+		m_dbusServiceExists = false;
+	}
+	else if(oldOwner.isEmpty())
+	{
+		m_dbusServiceExists = true;
 
-		if(newOwner.isEmpty())
-		{
-			m_dbusServiceExists = false;
+		// connect to action invocation signals
+		bool connected = QDBusConnection::sessionBus().connect(QString(), // from any service
+				dbusPath,
+				dbusInterfaceName,
+				"ActionInvoked",
+				this,
+				SLOT(slotDBusNotificationActionInvoked(uint,const QString&)));
+		if (!connected) {
+			kWarning() << "warning: failed to connect to ActionInvoked dbus signal";
 		}
-		else if(oldOwner.isEmpty())
-		{
-			m_dbusServiceExists = true;
 
-			// connect to action invocation signals
-			bool connected = QDBusConnection::sessionBus().connect(QString(), // from any service
-					dbusPath,
-					dbusInterfaceName,
-					"ActionInvoked",
-					this,
-					SLOT(slotDBusNotificationActionInvoked(uint,const QString&)));
-			if (!connected) {
-				kWarning(300) << "warning: failed to connect to ActionInvoked dbus signal";
-			}
-
-			connected = QDBusConnection::sessionBus().connect(QString(), // from any service
-					dbusPath,
-					dbusInterfaceName,
-					"NotificationClosed",
-					this,
-					SLOT(slotDBusNotificationClosed(uint,uint)));
-			if (!connected) {
-				kWarning(300) << "warning: failed to connect to NotificationClosed dbus signal";
-			}
+		connected = QDBusConnection::sessionBus().connect(QString(), // from any service
+				dbusPath,
+				dbusInterfaceName,
+				"NotificationClosed",
+				this,
+				SLOT(slotDBusNotificationClosed(uint,uint)));
+		if (!connected) {
+			kWarning() << "warning: failed to connect to NotificationClosed dbus signal";
 		}
 	}
 }
 
+
 void NotifyByPopup::slotDBusNotificationActionInvoked(uint dbus_id, const QString& actKey)
 {
 	// find out knotify id
 	int id = m_idMap.key(dbus_id, 0);
 	if (id == 0) {
-		kDebug(300) << "failed to find knotify id for dbus_id" << dbus_id;
+		kDebug() << "failed to find knotify id for dbus_id" << dbus_id;
 		return;
 	}
-	kDebug(300) << "action" << actKey << "invoked for notification " << id;
+	kDebug() << "action" << actKey << "invoked for notification " << id;
 	// emulate link clicking
 	slotLinkClicked( QString("%1/%2").arg(id).arg(actKey) );
     // now close notification - similar to popup behaviour
@@ -316,9 +324,9 @@ void NotifyByPopup::slotDBusNotificationClosed(uint dbus_id, uint reason)
 	Q_UNUSED(reason)
 	// find out knotify id
 	int id = m_idMap.key(dbus_id, 0);
-	kDebug(300) << dbus_id << "  -> " << id;
+	kDebug() << dbus_id << "  -> " << id;
 	if (id == 0) {
-		kDebug(300) << "failed to find knotify id for dbus_id" << dbus_id;
+		kDebug() << "failed to find knotify id for dbus_id" << dbus_id;
 		return;
 	}
 	// tell KNotify that this notification has been closed
@@ -333,7 +341,7 @@ void NotifyByPopup::getAppCaptionAndIconName(KNotifyConfig *config, QString *app
 	*iconName = globalgroup.readEntry("IconName", config->appname);
 }
 
-bool NotifyByPopup::sendNotificationDBus(int id, int replacesId, KNotifyConfig* config)
+bool NotifyByPopup::sendNotificationDBus(int id, int replacesId, KNotifyConfig* config_nocheck)
 {
 	// figure out dbus id to replace if needed
 	uint dbus_replaces_id = 0;
@@ -348,7 +356,9 @@ bool NotifyByPopup::sendNotificationDBus(int id, int replacesId, KNotifyConfig*
 	QList<QVariant> args;
 
 	QString appCaption, iconName;
-	getAppCaptionAndIconName(config, &appCaption, &iconName);
+	getAppCaptionAndIconName(config_nocheck, &appCaption, &iconName);
+
+	KNotifyConfig *config = ensurePopupCompatibility( config_nocheck );
 
 	args.append( appCaption ); // app_name
 	args.append( dbus_replaces_id ); // replaces_id
@@ -382,12 +392,15 @@ bool NotifyByPopup::sendNotificationDBus(int id, int replacesId, KNotifyConfig*
 
 	m.setArguments( args );
 	QDBusMessage replyMsg = QDBusConnection::sessionBus().call(m);
+
+	delete config;
+
 	if(replyMsg.type() == QDBusMessage::ReplyMessage) {
 		if (!replyMsg.arguments().isEmpty()) {
 			uint dbus_id = replyMsg.arguments().at(0).toUInt();
 			if (dbus_id == 0)
 			{
-				kDebug(300) << "error: dbus_id is null";
+				kDebug() << "error: dbus_id is null";
 				return false;
 			}
 			if (dbus_replaces_id && dbus_id == dbus_replaces_id) 
@@ -395,22 +408,22 @@ bool NotifyByPopup::sendNotificationDBus(int id, int replacesId, KNotifyConfig*
 #if 1
 			int oldId = m_idMap.key(dbus_id, 0);
 			if (oldId != 0) {
-				kWarning(300) << "Received twice the same id "<< dbus_id << "( previous notification: " << oldId << ")";
+				kWarning() << "Received twice the same id "<< dbus_id << "( previous notification: " << oldId << ")";
 				m_idMap.remove(oldId);
 				finish(oldId);
 			}
 #endif
 			m_idMap.insert(id, dbus_id);
-			kDebug(300) << "mapping knotify id to dbus id:"<< id << "=>" << dbus_id;
+			kDebug() << "mapping knotify id to dbus id:"<< id << "=>" << dbus_id;
 
 			return true;
 		} else {
-			kDebug(300) << "error: received reply with no arguments";
+			kDebug() << "error: received reply with no arguments";
 		}
 	} else if (replyMsg.type() == QDBusMessage::ErrorMessage) {
-		kDebug(300) << "error: failed to send dbus message";
+		kDebug() << "error: failed to send dbus message";
 	} else {
-		kDebug(300) << "unexpected reply type";
+		kDebug() << "unexpected reply type";
 	}
 	return false;
 }
@@ -419,7 +432,7 @@ void NotifyByPopup::closeNotificationDBus(int id)
 {
 	uint dbus_id = m_idMap.take(id);
 	if (dbus_id == 0) {
-		kDebug(300) << "not found dbus id to close" << id;
+		kDebug() << "not found dbus id to close" << id;
 	    return;
 	}
 
@@ -431,9 +444,90 @@ void NotifyByPopup::closeNotificationDBus(int id)
 	bool queued = QDBusConnection::sessionBus().send(m);
 	if(!queued)
 	{
-		kDebug(300) << "warning: failed to queue dbus message";
+		kDebug() << "warning: failed to queue dbus message";
 	}
 	
 }
 
+QStringList NotifyByPopup::popupServerCapabilities()
+{
+	if (!m_dbusServiceExists) {
+		// Return capabilities of the KPassivePopup implementation
+		return QStringList() << "actions" << "body" << "body-hyperlinks"
+		                     << "body-markup" << "icon-static";
+	}
+
+	if(m_dbusServiceCapCacheDirty) {
+		QDBusMessage m = QDBusMessage::createMethodCall( dbusServiceName, dbusPath,
+				                  dbusInterfaceName, "GetCapabilities" );
+		QDBusMessage replyMsg = QDBusConnection::sessionBus().call(m);
+		if (replyMsg.type() != QDBusMessage::ReplyMessage) {
+			kWarning(300) << "Error while calling popup server GetCapabilities()";
+			return QStringList();
+		}
+
+		if (replyMsg.arguments().isEmpty()) {
+			kWarning(300) << "popup server GetCapabilities() returned an empty reply";
+			return QStringList();
+		}
+
+		m_dbusServiceCapabilities = replyMsg.arguments().at(0).toStringList();
+		m_dbusServiceCapCacheDirty = false;
+	}
+
+	return m_dbusServiceCapabilities;
+}
+
+
+KNotifyConfig *NotifyByPopup::ensurePopupCompatibility( const KNotifyConfig *config )
+{
+	KNotifyConfig *c = config->copy();
+	QStringList cap = popupServerCapabilities();
+
+	if( !cap.contains( "actions" ) )
+	{
+		c->actions.clear();
+	}
+
+	if( !cap.contains( "body-markup" ) )
+	{
+		if( c->title.startsWith( "<html>" ) )
+			c->title = stripHtml( config->title );
+		if( c->text.startsWith( "<html>" ) )
+			c->text  = stripHtml( config->text  );
+	}
+
+	return c;
+}
+
+QString NotifyByPopup::stripHtml( const QString &text )
+{
+	QXmlStreamReader r( "<elem>" + text + "</elem>" );
+	QString result;
+	while( !r.atEnd() ) {
+		r.readNext();
+		if( r.tokenType() == QXmlStreamReader::Characters )
+		{
+			result.append( r.text() );
+		}
+		else if( r.tokenType() == QXmlStreamReader::StartElement
+		      && r.name()      == "br" )
+		{
+			result.append( "\n" );
+		}
+	}
+
+	if(r.hasError())
+	{
+		// XML error in the given text, just return the original string
+		kWarning(300) << "Notification to send to backend which does "
+		                 "not support HTML, contains invalid XML:"
+		              << r.errorString() << "line" << r.lineNumber()
+		              << "col" << r.columnNumber();
+		return text;
+	}
+
+	return result;
+}
+
 #include "notifybypopup.moc"
