@@ -47,7 +47,7 @@
 #define IPv6_PATTERN    "^\\[.*\\]"
 #define ENV_VAR_PATTERN "\\$[a-zA-Z_][a-zA-Z0-9_]*"
 
-#define QFL1(x) QLatin1String(x)
+#define QL1S(x) QLatin1String(x)
 
  /**
   * IMPORTANT:
@@ -64,15 +64,15 @@ typedef QMap<QString,QString> EntryMap;
 static QString stringDetails( const QString& str )
 {
   if ( str.isNull() )
-    return QFL1( "<null>" );
+    return QL1S( "<null>" );
   else if ( str.isEmpty() )
-    return QFL1( "<empty>" );
+    return QL1S( "<empty>" );
   else
     return str;
 }
 #endif
 
-static bool isValidShortURL( const QString& cmd, bool verbose = false )
+static bool isValidShortURL( const QString& cmd )
 {
   // Examples of valid short URLs:
   // "kde.org", "foo.bar:8080", "user@foo.bar:3128"
@@ -81,12 +81,10 @@ static bool isValidShortURL( const QString& cmd, bool verbose = false )
   QRegExp exp;
 
   // Match FQDN_PATTERN
-  exp.setPattern( QFL1(FQDN_PATTERN) );
+  exp.setPattern( QL1S(FQDN_PATTERN) );
   if ( cmd.contains( exp ) )
   {
-    if (verbose)
-      kDebug() << "KShortURIFilter::isValidShortURL: " << cmd
-                << " matches FQDN_PATTERN" << endl;
+    kDebug(7023) << cmd << " matches FQDN_PATTERN" << endl;
 #if 0
     // something like wallpaper.png also matches a the FQDN pattern
     // but is very unlikely to be meant as such
@@ -98,29 +96,22 @@ static bool isValidShortURL( const QString& cmd, bool verbose = false )
   }
 
   // Match IPv4 addresses
-  exp.setPattern( QFL1(IPv4_PATTERN) );
+  exp.setPattern( QL1S(IPv4_PATTERN) );
   if ( cmd.contains( exp ) )
   {
-    if (verbose)
-      kDebug() << "KShortURIFilter::isValidShortURL: " << cmd
-                << " matches IPv4_PATTERN" << endl;
+    kDebug(7023) << cmd << " matches IPv4_PATTERN" << endl;
     return true;
   }
 
   // Match IPv6 addresses
-  exp.setPattern( QFL1(IPv6_PATTERN) );
+  exp.setPattern( QL1S(IPv6_PATTERN) );
   if ( cmd.contains( exp ) )
   {
-    if (verbose)
-      kDebug() << "KShortURIFilter::isValidShortURL: " << cmd
-                << " matches IPv6_PATTERN" << endl;
+    kDebug(7023) << cmd << " matches IPv6_PATTERN" << endl;
     return true;
   }
 
-  if (verbose)
-    kDebug() << "KShortURIFilter::isValidShortURL: '" << cmd
-              << "' is not a short URL." << endl;
-
+  kDebug(7023) << cmd << "' is not a short URL." << endl;
   return false;
 }
 
@@ -141,7 +132,7 @@ static QString removeArgs( const QString& _cmd )
     if( spacePos > 0 )
     {
       cmd = cmd.left( spacePos );
-      //kDebug() << "spacePos=" << spacePos << " returning " << cmd;
+      //kDebug(7023) << "spacePos=" << spacePos << " returning " << cmd;
     }
   }
 
@@ -174,8 +165,16 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
   KUrl url = data.uri();
   QString cmd = data.typedString();
   const bool isMalformed = !url.isValid();
-  const QString protocol = url.protocol();
-  //kDebug() << "KShortUriFilter: url=" << url.url() << " cmd=" << cmd << " isMalformed=" << isMalformed;
+  QString protocol = url.protocol();
+
+  // Fix misparsing of "foo:80", QUrl thinks "foo" is the protocol and "80" is the path.
+  // However, be careful not to do that for valid hostless URLs, e.g. file:///foo!
+  if (!protocol.isEmpty() && url.host().isEmpty() && !url.path().isEmpty()
+      && cmd.contains(':') && !KProtocolInfo::protocols().contains(protocol)) {
+    protocol.clear();
+  }
+
+  //kDebug(7023) << "url=" << url << "cmd=" << cmd << "isMalformed=" << isMalformed;
 
   if (!isMalformed &&
       (protocol.length() == 4) &&
@@ -196,7 +195,7 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
   // executable and only the argument part, if any, changed! (Dawit)
   // You mean caching the last filtering, to try and reuse it, to save stat()s? (David)
 
-  const QString starthere_proto = QFL1("start-here:");
+  const QString starthere_proto = QL1S("start-here:");
   if (cmd.indexOf(starthere_proto) == 0 )
   {
     setFilteredUri( data, KUrl("system:/") );
@@ -205,16 +204,16 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
   }
 
   // Handle MAN & INFO pages shortcuts...
-  const QString man_proto = QFL1("man:");
-  const QString info_proto = QFL1("info:");
+  const QString man_proto = QL1S("man:");
+  const QString info_proto = QL1S("info:");
   if( cmd[0] == '#' ||
       cmd.indexOf( man_proto ) == 0 ||
       cmd.indexOf( info_proto ) == 0 )
   {
-    if( cmd.left(2) == QFL1("##") )
-      cmd = QFL1("info:/") + cmd.mid(2);
+    if( cmd.left(2) == QL1S("##") )
+      cmd = QL1S("info:/") + cmd.mid(2);
     else if ( cmd[0] == '#' )
-      cmd = QFL1("man:/") + cmd.mid(1);
+      cmd = QL1S("man:/") + cmd.mid(1);
 
     else if ((cmd==info_proto) || (cmd==man_proto))
       cmd+='/';
@@ -245,21 +244,21 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
 
   if (KUrl::isRelativeUrl(cmd) && QDir::isRelativePath(cmd)) {
      path = cmd;
-     //kDebug() << "path=cmd=" << path;
+     //kDebug(7023) << "path=cmd=" << path;
   } else {
     if (url.isLocalFile())
     {
-      //kDebug() << "hasRef=" << url.hasRef();
+      //kDebug(7023) << "hasRef=" << url.hasRef();
       // Split path from ref/query
       // but not for "/tmp/a#b", if "a#b" is an existing file,
       // or for "/tmp/a?b" (#58990)
       if( ( url.hasRef() || !url.query().isEmpty() )
-           && !url.path().endsWith(QFL1("/")) ) // /tmp/?foo is a namefilter, not a query
+           && !url.path().endsWith(QL1S("/")) ) // /tmp/?foo is a namefilter, not a query
       {
         path = url.path();
         ref = url.ref();
-        //kDebug() << "isLocalFile set path to " << stringDetails( path );
-        //kDebug() << "isLocalFile set ref to " << stringDetails( ref );
+        //kDebug(7023) << "isLocalFile set path to " << stringDetails( path );
+        //kDebug(7023) << "isLocalFile set ref to " << stringDetails( ref );
         query = url.query();
         if (path.isEmpty() && url.hasHost())
           path = '/';
@@ -267,7 +266,7 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
       else
       {
         path = cmd;
-        //kDebug() << "(2) path=cmd=" << path;
+        //kDebug(7023) << "(2) path=cmd=" << path;
       }
     }
   }
@@ -304,7 +303,7 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
   }
   else if ( path[0] == '$' ) {
     // Environment variable expansion.
-    QRegExp r (QFL1(ENV_VAR_PATTERN));
+    QRegExp r (QL1S(ENV_VAR_PATTERN));
     if ( r.indexIn( path ) == 0 )
     {
       const char* exp = getenv( path.mid( 1, r.matchedLength() - 1 ).toLocal8Bit().data() );
@@ -328,7 +327,7 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
       {
         ref = path.mid( pos + 1 );
         path = newPath;
-        //kDebug() << "Extracted ref: path=" << path << " ref=" << ref;
+        //kDebug(7023) << "Extracted ref: path=" << path << " ref=" << ref;
       }
     }
   }
@@ -345,12 +344,11 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
   const bool canBeLocalAbsolute = (canBeAbsolute && abs_path[0] =='/');
   bool exists = false;
 
-  //kDebug() << "abs_path=" << abs_path
-  //         << " protocol=" << protocol
-  //         << " canBeAbsolute=" << canBeAbsolute
-  //         << " canBeLocalAbsolute=" << canBeLocalAbsolute
-  //         << " isLocalFullPath=" << isLocalFullPath
-  //         << endl;
+  /*kDebug(7023) << "abs_path=" << abs_path
+               << "protocol=" << protocol
+               << "canBeAbsolute=" << canBeAbsolute
+               << "canBeLocalAbsolute=" << canBeLocalAbsolute
+               << "isLocalFullPath=" << isLocalFullPath;*/
 
   struct stat buff;
   if ( canBeLocalAbsolute )
@@ -360,9 +358,9 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
     int len = path.length();
     if( (len==1 && path[0]=='.') || (len==2 && path[0]=='.' && path[1]=='.') )
         path += '/';
-    //kDebug() << "adding " << abs << " and " << path;
+    //kDebug(7023) << "adding " << abs << " and " << path;
     abs = QDir::cleanPath(abs + '/' + path);
-    //kDebug() << "checking whether " << abs << " exists.";
+    //kDebug(7023) << "checking whether " << abs << " exists.";
     // Check if it exists
     if( stat( QFile::encodeName(abs).data(), &buff ) == 0 )
     {
@@ -388,7 +386,7 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
            && stat( QFile::encodeName(testPath).data(), &buff ) == 0 )
         {
           nameFilter = fileName;
-          //kDebug() << "Setting nameFilter to " << nameFilter;
+          //kDebug(7023) << "Setting nameFilter to " << nameFilter;
           path = testPath;
           exists = true;
         }
@@ -396,12 +394,12 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
     }
   }
 
-  //kDebug() << "path =" << path << " isLocalFullPath=" << isLocalFullPath << " exists=" << exists;
+  //kDebug(7023) << "path =" << path << " isLocalFullPath=" << isLocalFullPath << " exists=" << exists;
   if( exists )
   {
     KUrl u;
     u.setPath(path);
-    //kDebug() << "ref=" << stringDetails(ref) << " query=" << stringDetails(query);
+    //kDebug(7023) << "ref=" << stringDetails(ref) << " query=" << stringDetails(query);
     u.setRef(ref);
     u.setQuery(query);
 
@@ -418,7 +416,7 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
     bool isDir = S_ISDIR( buff.st_mode );
     if( !isDir && access ( QFile::encodeName(path).data(), X_OK) == 0 )
     {
-      //kDebug() << "Abs path to EXECUTABLE";
+      //kDebug(7023) << "Abs path to EXECUTABLE";
       setFilteredUri( data, u );
       setUriType( data, KUriFilterData::Executable );
       return true;
@@ -427,7 +425,7 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
     // Open "uri" as file:/xxx if it is a non-executable local resource.
     if( isDir || S_ISREG( buff.st_mode ) )
     {
-      //kDebug() << "Abs path as local file or directory";
+      //kDebug(7023) << "Abs path as local file or directory";
       if ( !nameFilter.isEmpty() )
         u.setFileName( nameFilter );
       setFilteredUri( data, u );
@@ -436,7 +434,7 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
     }
 
     // Should we return LOCAL_FILE for non-regular files too?
-    kDebug() << "File found, but not a regular file nor dir... socket?";
+    kDebug(7023) << "File found, but not a regular file nor dir... socket?";
   }
 
   // Let us deal with possible relative URLs to see
@@ -444,10 +442,10 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
   // We try hard to avoid parsing any possible command
   // line arguments or options that might have been supplied.
   QString exe = removeArgs( cmd );
-  //kDebug() << "findExe with " << exe;
+  //kDebug(7023) << "findExe with" << exe;
   if( data.checkForExecutables() && !KStandardDirs::findExe( exe ).isNull() )
   {
-    //kDebug() << "EXECUTABLE  exe=" << exe;
+    //kDebug(7023) << "EXECUTABLE  exe=" << exe;
     setFilteredUri( data, KUrl::fromPath( exe ));
     // check if we have command line arguments
     if( exe != cmd )
@@ -461,11 +459,11 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
   // slow things down...
   if ( !isMalformed && !isLocalFullPath && !protocol.isEmpty() )
   {
-    //kDebug() << "looking for protocol " << protocol;
+    //kDebug(7023) << "looking for protocol " << protocol;
     if ( KProtocolInfo::protocols().contains( protocol ) )
     {
       setFilteredUri( data, url );
-      if ( protocol == QFL1("man") || protocol == QFL1("help") )
+      if ( protocol == QL1S("man") || protocol == QL1S("help") )
         setUriType( data, KUriFilterData::Help );
       else
         setUriType( data, KUriFilterData::NetProtocol );
@@ -484,7 +482,7 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
       QRegExp match( (*it).regexp );
       if ( match.indexIn( cmd ) == 0 )
       {
-        //kDebug() << "match - prepending " << (*it).prepend;
+        //kDebug(7023) << "match - prepending" << (*it).prepend;
         cmd.prepend( (*it).prepend );
         setFilteredUri( data, KUrl( cmd ) );
         setUriType( data, (*it).type );
@@ -494,11 +492,10 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
 
     // If cmd is NOT a local resource, check if it is a valid "shortURL"
     // candidate and append the default protocol the user supplied. (DA)
-    if ( protocol.isEmpty() && isValidShortURL(cmd, m_bVerbose) )
+    if ( protocol.isEmpty() && isValidShortURL(cmd) )
     {
-      if (m_bVerbose)
-        kDebug() << "Valid short url, from malformed url -> using default proto="
-                  << m_strDefaultProtocol << endl;
+      kDebug(7023) << "Valid short url, from malformed url -> using default proto="
+               << m_strDefaultProtocol << endl;
 
       cmd.insert( 0, m_strDefaultProtocol );
       setFilteredUri( data, KUrl( cmd ));
@@ -523,7 +520,7 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
       setUriType( data, KUriFilterData::LocalFile );
       return true;
     }
-    //kDebug() << "fileNotFound -> ERROR";
+    //kDebug(7023) << "fileNotFound -> ERROR";
     setErrorMsg( data, i18n( "<qt>The file or folder <b>%1</b> does not exist.</qt>", data.uri().prettyUrl() ) );
     setUriType( data, KUriFilterData::Error );
     return true;
@@ -547,16 +544,12 @@ QString KShortUriFilter::configName() const
 
 void KShortUriFilter::configure()
 {
-  KConfig config( objectName() + QFL1( "rc"), KConfig::NoGlobals );
+  KConfig config( objectName() + QL1S( "rc"), KConfig::NoGlobals );
   KConfigGroup cg( config.group("") );
-  m_bVerbose = cg.readEntry( "Verbose", false );
-
-  if ( m_bVerbose )
-    kDebug() << "KShortUriFilter::configure: Config reload request...";
 
   m_strDefaultProtocol = cg.readEntry( "DefaultProtocol", QString("http://") );
-  const EntryMap patterns = config.entryMap( QFL1("Pattern") );
-  const EntryMap protocols = config.entryMap( QFL1("Protocol") );
+  const EntryMap patterns = config.entryMap( QL1S("Pattern") );
+  const EntryMap protocols = config.entryMap( QL1S("Protocol") );
   KConfigGroup typeGroup(&config, "Type");
 
   for( EntryMap::ConstIterator it = patterns.begin(); it != patterns.end(); ++it )
