@@ -32,6 +32,7 @@
 #include <QtCore/QDateTime>
 #include <QtCore/QByteArray>
 #include <QtCore/QUrl>
+#include <QtCore/QTimer>
 
 #include <KDebug>
 #include <KTemporaryFile>
@@ -63,21 +64,39 @@ namespace {
     const int s_reducedSpeedDelay = 500; // ms
     const int s_snailPaceDelay = 3000;   // ms
 
+    bool isResourcePresent( const QString & dir ) {
+        QString query = QString::fromLatin1(" ask { ?r %1 %2. } ")
+                        .arg( Soprano::Node::resourceToN3( Nepomuk::Vocabulary::NIE::url() ),
+                              Soprano::Node::resourceToN3( KUrl( dir ) ) );
+        return Nepomuk::ResourceManager::instance()->mainModel()->executeQuery( query, Soprano::Query::QueryLanguageSparql ).boolValue();
+    }
 
     QHash<QString, QDateTime> getChildren( const QString& dir )
     {
         QHash<QString, QDateTime> children;
+        QString query;
 
-        QString query = QString( "select distinct ?url ?mtime where { "
-                                 "?r %1 ?parent . ?parent %2 %3 . "
-                                 "?r %4 ?mtime . "
-                                 "?r %2 ?url . "
-                                 "}")
-                        .arg( Soprano::Node::resourceToN3( Nepomuk::Vocabulary::NIE::isPartOf() ),
-                              Soprano::Node::resourceToN3( Nepomuk::Vocabulary::NIE::url() ),
-                              Soprano::Node::resourceToN3( KUrl( dir ) ),
-                              Soprano::Node::resourceToN3( Nepomuk::Vocabulary::NIE::lastModified() ) );
-
+        if( !isResourcePresent( dir ) ) {
+            query = QString::fromLatin1( "select distinct ?url ?mtime where { "
+                                         "?r %1 ?url . "
+                                         "FILTER( regex(str(?url), '^file://%2/([^/]*)$') ) . "
+                                         "?r %3 ?mtime ."
+                                         "}" )
+                    .arg( Soprano::Node::resourceToN3( Nepomuk::Vocabulary::NIE::url() ),
+                          dir,
+                          Soprano::Node::resourceToN3( Nepomuk::Vocabulary::NIE::lastModified() ) );
+        }
+        else {
+            query = QString::fromLatin1( "select distinct ?url ?mtime where { "
+                                        "?r %1 ?parent . ?parent %2 %3 . "
+                                        "?r %4 ?mtime . "
+                                        "?r %2 ?url . "
+                                        "}" )
+                    .arg( Soprano::Node::resourceToN3( Nepomuk::Vocabulary::NIE::isPartOf() ),
+                        Soprano::Node::resourceToN3( Nepomuk::Vocabulary::NIE::url() ),
+                        Soprano::Node::resourceToN3( KUrl( dir ) ),
+                        Soprano::Node::resourceToN3( Nepomuk::Vocabulary::NIE::lastModified() ) );
+        }
         //kDebug() << "running getChildren query:" << query;
 
         Soprano::QueryResultIterator result = Nepomuk::ResourceManager::instance()->mainModel()->executeQuery( query, Soprano::Query::QueryLanguageSparql );
@@ -130,6 +149,13 @@ Nepomuk::IndexScheduler::IndexScheduler( Strigi::IndexManager* manager, QObject*
 {
     m_analyzerConfig = new StoppableConfiguration;
 
+    // see updateDir(QString,bool) for details on the timer
+    m_dirsToUpdateWakeupTimer = new QTimer( this );
+    m_dirsToUpdateWakeupTimer->setSingleShot( true );
+    m_dirsToUpdateWakeupTimer->setInterval(1000);
+    connect( m_dirsToUpdateWakeupTimer, SIGNAL( timeout() ),
+             this, SLOT( slotDirsToUpdateWakeupTimeout() ) );
+
     connect( StrigiServiceConfig::self(), SIGNAL( configChanged() ),
              this, SLOT( slotConfigChanged() ) );
 }
@@ -383,7 +409,9 @@ bool Nepomuk::IndexScheduler::updateDir( const QString& dir, Strigi::StreamAnaly
         if ( !waitForContinue() )
             return false;
 
+        m_currentUrl = file.filePath();
         analyzeFile( file, analyzer );
+        m_currentUrl = KUrl();
     }
 
     // recurse into subdirs (we do this in a separate loop to always keep a proper state:
@@ -405,7 +433,6 @@ void Nepomuk::IndexScheduler::analyzeFile( const QFileInfo& file, Strigi::Stream
     //
     // strigi asserts if the file path has a trailing slash
     //
-    m_currentUrl = file.filePath();
     QString filePath = m_currentUrl.toLocalFile( KUrl::RemoveTrailingSlash );
     QString dir = m_currentUrl.directory(KUrl::IgnoreTrailingSlash);
 
@@ -427,9 +454,6 @@ void Nepomuk::IndexScheduler::analyzeFile( const QFileInfo& file, Strigi::Stream
     else {
         analysisresult.index(0);
     }
-
-    // done with this file
-    m_currentUrl = KUrl();
 }
 
 
@@ -453,7 +477,15 @@ void Nepomuk::IndexScheduler::updateDir( const QString& path, bool forceUpdate )
 {
     QMutexLocker lock( &m_dirsToUpdateMutex );
     m_dirsToUpdate << qMakePair( path, UpdateDirFlags( forceUpdate ? ForceUpdate : NoUpdateFlags ) );
-    m_dirsToUpdateWc.wakeAll();
+
+    // sometimes the filewatch service will call this method many times in a row with the same
+    // folder (in case a whole folder is created or modified). In order not to run an update every
+    // time we slightly delay the update process
+    if ( !isSuspended() &&
+         isIndexing() &&
+         !m_dirsToUpdateWakeupTimer->isActive() ) {
+        m_dirsToUpdateWakeupTimer->start();
+    }
 }
 
 
@@ -495,6 +527,12 @@ void Nepomuk::IndexScheduler::slotConfigChanged()
 }
 
 
+void Nepomuk::IndexScheduler::slotDirsToUpdateWakeupTimeout()
+{
+    m_dirsToUpdateWc.wakeAll();
+}
+
+
 namespace {
     class QDataStreamStrigiBufferedStream : public Strigi::BufferedStream<char>
     {
@@ -545,6 +583,14 @@ void Nepomuk::IndexScheduler::analyzeResource( const QUrl& uri, const QDateTime&
 }
 
 
+void Nepomuk::IndexScheduler::analyzeFile( const QString& path )
+{
+    Strigi::StreamAnalyzer analyzer( *m_analyzerConfig );
+    analyzer.setIndexWriter( *m_indexManager->indexWriter() );
+    analyzeFile( path, &analyzer );
+}
+
+
 void Nepomuk::IndexScheduler::deleteEntries( const QStringList& entries )
 {
     // recurse into subdirs
@@ -629,6 +675,8 @@ void Nepomuk::IndexScheduler::removeOldAndUnwantedEntries()
     // be indexed at once.
     //
     QString folderFilter = constructFolderFilter();
+    if( !folderFilter.isEmpty() )
+        folderFilter = QString::fromLatin1("FILTER(%1) .").arg(folderFilter);
 
     //
     // We query all files that should not be in the store
@@ -638,7 +686,7 @@ void Nepomuk::IndexScheduler::removeOldAndUnwantedEntries()
                                          "?r %1 ?url . "
                                          "?g <http://www.strigi.org/fields#indexGraphFor> ?r . "
                                          "FILTER(REGEX(STR(?url),'^file:/')) . "
-                                         "FILTER(%2) . }" )
+                                         "%2 }" )
                     .arg( Soprano::Node::resourceToN3( Nepomuk::Vocabulary::NIE::url() ),
                           folderFilter );
     kDebug() << query;
@@ -660,20 +708,29 @@ void Nepomuk::IndexScheduler::removeOldAndUnwantedEntries()
     //
     // Build filter query for all exclude filters
     //
-    QStringList filters;
+    QStringList fileFilters;
     foreach( const QRegExp& re, Nepomuk::StrigiServiceConfig::self()->excludeFilterRegExps() ) {
-        filters << QString::fromLatin1( "REGEX(STR(?fn),\"/%1$\")" ).arg( re.pattern().replace( '\\',"\\\\" ) );
+        fileFilters << QString::fromLatin1( "REGEX(STR(?fn),\"^%1$\")" ).arg( re.pattern().replace( '\\',"\\\\" ) );
     }
+    QString includeExcludeFilters = constructExcludeIncludeFoldersFilter();
+
+    QString filters;
+    if( !includeExcludeFilters.isEmpty() && !fileFilters.isEmpty() )
+        filters = QString::fromLatin1("FILTER((%1) && (%2)) .").arg( includeExcludeFilters, fileFilters.join(" || ") );
+    else if( !fileFilters.isEmpty() )
+        filters = QString::fromLatin1("FILTER(%1) .").arg( fileFilters.join(" || ") );
+    else if( !includeExcludeFilters.isEmpty() )
+        filters = QString::fromLatin1("FILTER(%1) .").arg( includeExcludeFilters );
+    
     query = QString::fromLatin1( "select distinct ?g ?url where { "
                                  "?r %1 ?url . "
                                  "?r %2 ?fn . "
                                  "?g <http://www.strigi.org/fields#indexGraphFor> ?r . "
-                                 "FILTER(REGEX(STR(?url),'^file:/')) . "
-                                 "FILTER((%3) && (%4)) . }" )
+                                 "FILTER(REGEX(STR(?url),\"^file:/\")) . "
+                                 "%3 }" )
             .arg( Soprano::Node::resourceToN3( Nepomuk::Vocabulary::NIE::url() ),
                   Soprano::Node::resourceToN3( Nepomuk::Vocabulary::NFO::fileName() ),
-                  constructExcludeIncludeFoldersFilter(),
-                  filters.join( " || " ) );
+                  filters );
     kDebug() << query;
     it = ResourceManager::instance()->mainModel()->executeQuery( query, Soprano::Query::QueryLanguageSparql );
     while ( it.next() ) {
